/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "commandline.h"
#include "html.h"
#include "stylesheet.h"
#include "exception.h"
#include "ssf.h"
#include "scripture.h"
#include "mainwindow.h"
#include "book.h"


int main(int argc, char *argv[]) {
  set_terminate(terminator);	// Unhandled exception handler.
  set_unexpected(my_unexpected); // Handler for unexpected exceptions.

  // No arguments: start the GUI
  if (argc == 1) {
    Glib::thread_init();
    Gtk::Main kit(argc, argv);
    MainWindow mainwindow;
    Gtk::Main::run(mainwindow);
  } else {
    // Declare a host of variables.
    bool runhtml = false;
    bool loadstylesheet = false;
    bool readshortcuts = false;
    bool readfullname = false;
    bool readdirectory = false;
    bool readnaming = false;
    bool readfontname = false;
    bool readfontsize = false;
    bool readversification = false;
    bool readstylesheet = false;
    bool readlanguagefontname = false;
    bool readlanguagefontsize = false;
    bool readbookfilenames = false;
    bool readbookids = false;
    bool readbookenglishnames = false;
    bool readnumberofchapters = false;
    bool readnumberofverses = false;
    bool readachapter = false;
    bool saveachapter = false;
    bool sortbackups = false;
    bool dumpbook = false;
    string input;
    string output;
    string stylesheet;
    string projectdirectory;
    unsigned int chapter;
    // Go through the arguments on the commandline, and set the appropriate variables.
    for (int i = 1; i < argc; i++) {
      // argv[i] cannot be compared using the == operator, only strings
      // do this. So assign argv[i] first to a C++ string object.
      string s = argv[i];
      if (s == OPTION_HTML) {
	runhtml = true;
      }  
      else if (s == OPTION_INPUT) {
	i++;
	if (i < argc)
	  input = argv[i];
	if (i >= argc) {
	  cerr << "No input name given" << endl;
	  throw;
	}
      }
      else if (s == OPTION_OUTPUT) {
	i++;
	if (i < argc)
	  output = argv[i];
	if (i >= argc) {
	  cerr << "No output name given" << endl;
	  throw;
	}
      }
      else if (s == OPTION_STYLESHEET) {
	loadstylesheet = true;
	i++;
	if (i < argc)
	  stylesheet = argv[i];
	if (i >= argc) {
	  cerr << "No stylesheet given" << endl;
	  throw;
	}
      }
      else if (s == OPTION_READSHORTCUTS) {
	readshortcuts = true;
      }
      else if (s == OPTION_PROJECTDIRECTORY) {
	i++;
	if (i < argc)
	  projectdirectory = argv[i];
	if (i >= argc) {
	  cerr << "No projectdirectory given" << endl;
	  throw;
	}
      }
      else if (s == OPTION_READFULLNAME) {
	readfullname = true;
      }
      else if (s == OPTION_READDIRECTORY) {
	readdirectory = true;
      }
      else if (s == OPTION_READNAMING) {
	readnaming = true;
      }
      else if (s == OPTION_READFONTNAME) {
	readfontname = true;
      }
      else if (s == OPTION_READFONTSIZE) {
	readfontsize = true;
      }
      else if (s == OPTION_READVERSIFICATION) {
	readversification = true;
      }
      else if (s == OPTION_READSTYLESHEET) {
	readstylesheet = true;
      }
      else if (s == OPTION_READLANGUAGEFONTNAME) {
	readlanguagefontname = true;
      }
      else if (s == OPTION_READLANGUAGEFONTSIZE) {
	readlanguagefontsize = true;
      }
      else if (s == OPTION_READBOOKFILENAMES) {
	readbookfilenames = true;
      }
      else if (s == OPTION_READBOOKIDS) {
	readbookids = true;
      }
      else if (s == OPTION_READBOOKENGLISHNAMES) {
	readbookenglishnames = true;
      }
      else if (s == OPTION_READNUMBEROFCHAPTERS) {
	readnumberofchapters = true;
      }
      else if (s == OPTION_READNUMBEROFVERSES) {
	readnumberofverses = true;
      }
      else if (s == OPTION_READACHAPTER) {
	readachapter = true;
      }
      else if (s == OPTION_SAVEACHAPTER) {
	saveachapter = true;
      }
      else if (s == OPTION_SORTBACKUPS) {
	sortbackups = true;
      }
      else if (s == OPTION_DUMPBOOK) {
	dumpbook = true;
      }
      else if (s == OPTION_CHAPTER) {
	i++;
	if (i < argc)
	  chapter = convert_to_int(argv[i]);
	if (i >= argc) {
	  cerr << "No chapter given" << endl;
	  throw;
	}
      }
      else {
	cerr << s << " is an unknown argument" << endl;
      }
    }
    // Carry out all the commands.
    // Stylesheet should be loaded before tasks that need it.
    Stylesheet sheet;
      if (loadstylesheet) {
      sheet.readsheet (stylesheet);
    }
    if (runhtml) {
      HtmlConverter hc (sheet);
      hc.convert (input, output);
    }
    if (readshortcuts) {
      create_directory (Glib::path_get_dirname(output));
      ScriptureSettingsFile ssf (projectdirectory);
      write_lines (output, ssf.shortcuts);
    }
    if (readfullname || readdirectory || readfontname || readstylesheet || readlanguagefontname) {
      ScriptureSettingsFile ssf (projectdirectory);
      string s;
      unsigned int i = convert_to_int (input);
      if (readfullname) s = ssf.FullName(i);
      if (readdirectory) s = ssf.Directory(i);
      if (readfontname) s = ssf.FontName(i);
      if (readstylesheet) s = ssf.StyleSheet(i);
      if (readlanguagefontname) s = ssf.LDSFontName(i);
      WriteText wt(output);
      wt.out << s << endl;
    }
    if (readnaming) {
      unsigned int i = convert_to_int(input);
      ScriptureSettingsFile ssf (projectdirectory);
      string pre;
      string post;
      string form;
      ssf.Naming (i, pre, post,form);
      WriteText wt(output);
      wt.out << pre << endl;
      wt.out << post << endl;
      wt.out << form << endl;
    }
    if (readfontsize || readversification || readlanguagefontsize) {
      ScriptureSettingsFile ssf (projectdirectory);
      unsigned int value;
      unsigned int i = convert_to_int (input);
      if (readfontsize) value = ssf.FontSize(i);
      if (readversification) value = ssf.Versification(i);
      if (readlanguagefontsize) value = ssf.LDSFontSize(i);
      WriteText wt(output);
      wt.out << value << endl;
    }
    if (readbookfilenames || readbookids || readbookenglishnames) {
      create_directory (Glib::path_get_dirname(output));
      Scripture s(0);
      s.load (projectdirectory, convert_to_int (input));
      if (readbookfilenames) write_lines (output, s.FilePathAndNames);
      if (readbookids) write_lines (output, s.IDs);
      if (readbookenglishnames) write_lines (output, s.EnglishBookNames);
    }
    if (readnumberofchapters || readnumberofverses || readachapter || saveachapter) {
      Book book(0);
      book.load(input, sheet, false, "");
      WriteText wt(output);
      if (readnumberofchapters) wt.out << book.number_of_chapters << endl;
      if (readnumberofverses) wt.out << book.number_of_verses(chapter) << endl;
      vector<string> lines;
      if (readachapter) {
	book.get_chapter(chapter, lines);
	for (unsigned int i = 0; i < lines.size(); i++) {
	  wt.out << lines[i] << endl;
	}
      }
      if (saveachapter) {
	book.get_chapter(chapter, lines);
	lines.erase(lines.begin());
	lines.erase(++lines.begin());
	lines.push_back("Added1");
	lines.push_back("Added2");
	book.set_chapter(lines);
	book.save();
      }
    }
    if (sortbackups) {
      Book book(0);
      book.load(input, sheet, true, input);
      Backup backup(0);
      vector<string> backups = book.get_backups();
      backup.load (input, backups);
      backup.sort_out_old_backups();
    }
    if (dumpbook) {
      Book book(0);
      book.load(input, sheet, false, "");
      vector<string> lines;
      book.dump_with_references(lines);
      write_lines(output, lines);
    }
  }
  return 0;  
}


