/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogconvert2web.h"
#include "ssf.h"
#include "utilities.h"
#include "constants.h"
#include "scripture.h"
#include "dialogscript.h"


#define CONVERTTEMPFILE "bibledit.convert"


ConvertToWebDialog::ConvertToWebDialog(Gtk::Window& parent, Configuration& configuration) :
  Dialog ("Convert a project to web files", parent, true, false),
  button2("Convert")
{
  converttowebdirectory = configuration.converttowebdirectory;
  projectdirectory = configuration.projectdirectory;
  projectshortcut = configuration.projectshortcut;
  
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  label1.set_text("Converting a project to web files consist of the following steps.\n" \
		 "1. Indicating the directory where the web files are to be placed.\n" \
		 "   (this is not needed every time, only when there is a change)\n" \
		 "2. Converting the project.");
  label1.set_padding(10, 10);
  get_vbox()->pack_start(label1, false, false, 0);

  button1.signal_clicked().connect(SigC::slot(*this, &ConvertToWebDialog::on_button1_clicked));
  get_vbox()->pack_start(button1, false, false, 10);

  button2.signal_clicked().connect(SigC::slot(*this, &ConvertToWebDialog::on_button2_clicked));
  get_vbox()->pack_start(button2, false, false, 10);

  set_gui();
  show_all_children();
  button2.grab_focus();
}


ConvertToWebDialog::~ConvertToWebDialog() {
  unlink(temporary_file(CONVERTTEMPFILE).c_str());	// Remove the temporary file that may have been used.
}


void ConvertToWebDialog::on_button1_clicked() {
  Gtk::FileSelection dialog("Please choose a folder");
  dialog.set_transient_for(*this);
  dialog.get_file_list()->get_parent()->hide(); //Prevent the user from selecting a file.
  dialog.hide_fileop_buttons();	// Hide the buttons that operate on files.
  dialog.set_filename(converttowebdirectory + "/");
  int result = dialog.run();
  if (result == Gtk::RESPONSE_OK) {  //Handle the response:
    converttowebdirectory = dialog.get_filename();
    set_gui();
  }
}


void ConvertToWebDialog::on_button2_clicked() {
  ScriptureSettingsFile ssf (projectdirectory);
  for (unsigned int i = 0; i < ssf.shortcuts.size(); i++) {
    if (projectshortcut == ssf.shortcuts[i]) {
      Scripture scripture(0);
      scripture.load(projectdirectory, i);
      WriteText wt(temporary_file(CONVERTTEMPFILE)); // Produce the file that contains the names of the files to be converted ...
      for (unsigned int i2 = 0; i2 < scripture.FilePathAndNames.size(); i2++) {
	wt.out << scripture.FilePathAndNames[i2] << endl;
      }
      // Produce the command to run the script.
      string command = "bibledit --html";                // Name of the script and argument.
      command.append(" --input ");                       // The input command, with the actual input value
      command.append("'");	                         // This value is in single quotes, so that the shell
      command.append(temporary_file (CONVERTTEMPFILE));  // will do no filename globbing or cutting the value.
      command.append("'");
      command.append(" --output ");                      // The output command, with the actual output value
      command.append("'");	                         // This value is in single quotes, so that the shell
      command.append(converttowebdirectory);             // will do no filename globbing or cutting of the value.
      command.append("'");
      command.append(" --stylesheet ");                  // The stylesheet command, with the actual stylesheet value
      command.append("'");	                         // This value is in single quotes, so that the shell
      command.append(ssf.StyleSheet(i));	         // will do no filename globbing or cutting of the value.
      command.append("'");
      command.append(" 2>&1");	                         // cerr goes into cout
      ScriptDialog scriptdialog (*this, command);
      scriptdialog.run();
    }
  }
}


void ConvertToWebDialog:: set_gui() {
  // This sets the labels etc. of the dialog.
  button1.set_label("  Indicate directory for the web files: " + converttowebdirectory + "  ");
}


#undef CONVERTTEMPFILE
