/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogfind.h"
#include "scripture.h"
#include "ssf.h"
#include "utilities.h"
#include "dialogscript.h"


FindDialog::FindDialog(Gtk::Window& parent, const string& directory, const string& shortcut) :
  Dialog ("Find", parent, true, false)
{
  add_button(Gtk::Stock::FIND, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  set_default_response(Gtk::RESPONSE_OK);
  // Set up all the widgets
  vbox.pack_start(hbox);
  hbox.pack_start(label, 0, 0, 10);
  label.set_text("Search for");
  label.set_padding(0, 10);
  hbox.pack_start(entry, 0, 0, 10);
  get_vbox()->pack_start(vbox);
  // Focus the entry
  set_focus_child(entry);
  show_all_children();
  mydirectory = directory;
  myshortcut = shortcut;
}


FindDialog::~FindDialog() {
}


#define SCRIPTFILE "bibledit.find.script"
#define DUMPFILE "bibledit.find.dump"


void FindDialog::on_response (int response_id) {
  if (response_id == Gtk::RESPONSE_OK) {
    // Get some data about the books to search through.
    ScriptureSettingsFile ssf (mydirectory);
    unsigned int index = ssf.index(myshortcut);
    Scripture scripture(0);
    scripture.load(mydirectory, index);
    // Stylesheet is needed in the script.
    string stylesheet = ssf.StyleSheet(index);
    // Start to make a script.
    WriteText wt(temporary_file(SCRIPTFILE));
    wt.out << "rm -f " << temporary_file(FINDRESULTS) << endl;
    for (unsigned int i = 0; i < scripture.FilePathAndNames.size(); i++) {
      // Percentage indicator.
      wt.out << "echo " << scripture.EnglishBookNames[i] << " " << 100 * i / (scripture.EnglishBookNames.size() - 1) << "%" << endl;
      // Bibledit will dump book by book, with the references prefixed, e.g. MAT 1:1.
      wt.out << "bibledit --dumpbook";
      wt.out << " --input '" << scripture.FilePathAndNames[i] << "'";
      wt.out << " --output " << temporary_file(DUMPFILE);
      wt.out << " --stylesheet '" << stylesheet << "'";
      wt.out << endl;
      // Search through each file.
      wt.out << "grep '" << entry.get_text() << "' " << temporary_file(DUMPFILE) << " >> " << temporary_file(FINDRESULTS) << endl;
    }
    // If no results are found in the last book, the exit code of grep will be returned, as if there is an error.
    // Prevent this by adding a last command that always succeeds.
    wt.out << "echo Ready" << endl;
    // Produce command to run it.
    string command = "bash ";
    command.append(temporary_file(SCRIPTFILE));
    command.append(" 2>&1");	
    ScriptDialog scriptdialog (*this, command);
    scriptdialog.run();
    // Inform user if we have no results.
    command = "test -s " + temporary_file(FINDRESULTS);
    int result = system(command.c_str());
    if (result != 0) {
      Gtk::MessageDialog dialog(*this, "No search results");
      dialog.run();
    }
    // Remove temporary files used.
    unlink(temporary_file(SCRIPTFILE).c_str());
    unlink(temporary_file(DUMPFILE).c_str());
  }
}



// Todo 

/*

unit search. It has several functions for scripting, etc.

Add a "select books" window, with an easy way of selecting part or all of the book.

For selecting books, it uses set<string>, and then set.find() to find out if the book is in the set or not.

*/
