/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogopen.h"
#include "ssf.h"



OpenDialog::OpenDialog(Gtk::Window& parent, Configuration& configuration) :
  Dialog ("Open project", parent, true, false)
{
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  label1.set_text("Opening a project consists of two steps.\n" \
		 "1. Indicating the location of the directory with all the projects.\n" \
		 "   (this needs only be done once, because it does not usually change)\n" \
		 "2. Opening one of all the available projects in that directory.");
  label1.set_padding(10, 10);
  get_vbox()->pack_start(label1, false, false, 0);

  button.signal_clicked().connect(SigC::slot(*this, &OpenDialog::on_button_clicked));
  get_vbox()->pack_start(button, false, false, 10);

  projectdirectory = configuration.projectdirectory;
  projectshortcut = configuration.projectshortcut;
  readprojects();

  /*
    Calling the changed handler once is needed for a reason.
    If it would not be called, and the user would open a project,
    notice that he or she wants the first project listed, therefore
    would not change the combo box, and would then press OK to open
    the project, the project would not open in the editor. This is
    the project shortcut variabele would not have been initialised.
    So by callng the changed handler once, it gets initialised with
    the value currently being selected.
   */
  on_combo_changed();

  combo.get_entry()->signal_changed().connect( SigC::slot(*this, &OpenDialog::on_combo_changed) );
  combo.get_entry()->set_editable(false);
  get_vbox()->pack_start(combo, false, false, 10);

  show_all_children();

  // Focus is on the combobox, the selector of the projects. Convenient for the user.
  set_focus_child(combo);
}


OpenDialog::~OpenDialog() {
}


void OpenDialog::on_button_clicked() {
  Gtk::FileSelection dialog("Please choose a folder");
  dialog.set_transient_for(*this);
  dialog.get_file_list()->get_parent()->hide(); //Prevent the user from selecting a file.
  dialog.hide_fileop_buttons();	// Hide the buttons that operate on files.
  dialog.set_filename(projectdirectory + "/");
  int result = dialog.run();
  if (result == Gtk::RESPONSE_OK) {  //Handle the response:
    projectdirectory = dialog.get_filename();
    readprojects();
  }
}


void OpenDialog::on_combo_changed() {
  Glib::ustring text = combo.get_entry()->get_text();
  if(!text.empty()) { // We seem to get 2 signals, one when the text is empty.
    ScriptureSettingsFile ssh(projectdirectory);
    for (unsigned int i = 0; i < ssh.shortcuts.size(); i++) {
      if (ssh.FullName(i) == text)
	projectshortcut = ssh.shortcuts[i];
    }
  }
}


void OpenDialog::readprojects() {
  // This reads all the Paratext projects in the project directory,
  ScriptureSettingsFile ssf (projectdirectory);
  shortcuts.clear();
  names.clear();
  for (unsigned int i = 0; i < ssf.shortcuts.size(); i++) {
    shortcuts.push_back(ssf.shortcuts[i]);
    names.push_back(ssf.FullName(i));
  }
  if (ssf.shortcuts.size() == 0) {
    names.push_back("<No projects here>");
  }
  combo.set_popdown_strings(names);
  button.set_label("  Indicate directory with all projects: " + projectdirectory + "  ");
  // Now see whether the shortcut that "configuration" indicates can be translated to a text on the combo widget.
  ScriptureSettingsFile ssh(projectdirectory);
  for (unsigned int i = 0; i < ssh.shortcuts.size(); i++) {
    if (projectshortcut == ssh.shortcuts[i]) {
      combo.get_entry()->set_text(ssh.FullName(i));
    }
  }
}


