/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogsynchronization.h"
#include "ssf.h"
#include "utilities.h"


SynchronizationDialog::SynchronizationDialog(Gtk::Window& parent, Configuration& configuration) :
  Dialog ("Synchronization with Paratext", parent, true, false)
{
  directory = configuration.synchronize_paratext_projectdirectory;

  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  label1.set_text("To keep some files in the project directories of Bibledit and of Paratext synchronized,\n" \
		  "do the following:\n" \
		  "1. Enable the synchronization\n" \
		  "1. Select the Paratext project directory\n" \
		  "2. Enter the names of the files to be synchronized\n");
  label1.set_padding(5, 5);
  get_vbox()->pack_start(label1, false, false, 0);

  get_vbox()->pack_start(checkbutton, false, false, 5);
  checkbutton.set_label("Synchronize");
  checkbutton.set_active(configuration.synchronize_paratext);

  button.signal_clicked().connect(SigC::slot(*this, &SynchronizationDialog::on_button_clicked));
  get_vbox()->pack_start(button, false, false, 5);

  label2.set_padding(5, 5);
  get_vbox()->pack_start(label2, false, false, 0);

  label3.set_padding(5, 5);
  label3.set_text("In the area below, enter all the names of the files to be synchronized, one per line");
  get_vbox()->pack_start(label3, false, false, 0);
  /*
    With unison the names that are entered here should be the full filenames, including a path is there is one.
    Filename globbing does not work with this setting. It does work with "ignore" directives, which we don't use.
   */

  get_vbox()->pack_start(scrolledwindow);
  scrolledwindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  scrolledwindow.add(textview);
  textview.set_wrap_mode(Gtk::WRAP_WORD);
  try {
    string contents = Glib::file_get_contents(synchronization_paths_filename());
    textview.get_buffer()->set_text(contents);
  }
  catch(Glib::FileError) {
  }

  set_gui();

  show_all_children();

  // Focus is on the checkbutton. Convenient for the user.
  set_focus_child(checkbutton);
}


SynchronizationDialog::~SynchronizationDialog() {
}


void SynchronizationDialog::on_button_clicked() {
  Gtk::FileSelection dialog("Please choose a folder");
  dialog.set_transient_for(*this);
  dialog.get_file_list()->get_parent()->hide(); //Prevent the user from selecting a file.
  dialog.hide_fileop_buttons();	// Hide the buttons that operate on files.
  dialog.set_filename(directory + "/");
  int result = dialog.run();
  if (result == Gtk::RESPONSE_OK) {  //Handle the response:
    directory = dialog.get_filename();
    readprojects();
    set_gui();
  }
}


int SynchronizationDialog::readprojects() {
  /*
    This reads all the Paratext projects in the project directory.
    It returns the number of projects found there.
  */
  ScriptureSettingsFile ssf (directory);
  return ssf.shortcuts.size();
}


void SynchronizationDialog::set_gui() {
  button.set_label("  Indicate directory with the Paratext projects: " + directory + "  ");
  label2.set_text("This directory contains " + convert_to_string(readprojects()) + " projects");
}

