/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "log.h"
#include <libgen.h>
#include "utilities.h" 
#include <time.h>


void log_message (const string& message) {
  // Some static variables.
  static string previous_message;
  static unsigned int previous_message_counter = 0;
  // See whether this message is the same as the previous one.
  if (message.compare(previous_message) == 0) {
    previous_message_counter++;
    return;
  }
  // Get the current date/time.
  time_t value = time(0);
  string current_date  = ctime(&value);
  current_date = trim(current_date);
  // Opening the logfile for append.
  WriteText wt(temporary_file(LOGFILE), true);
  // First deal with any previous messages that were repeated.
  if (previous_message_counter > 0) {
    wt.out << current_date << " Previous message was repeated " << previous_message_counter << " times" << endl;
    previous_message_counter = 0;
  }
  // Append the message to the log file.
  wt.out << current_date << " " << message << endl;
  // Store previous message.
  previous_message = message;
}
