/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef INCLUDED_MAINWINDOW_H
#define INCLUDED_MAINWINDOW_H

#include <gtkmm.h>
#include "configuration.h"
#include "stylesheet.h"
#include "track.h"
#include "book.h"
#include "scripture.h"
#include "backup.h"
#include "undo.h"
#include "windowscompanion.h"


class MainWindow : public Gtk::Window
{
public:
  MainWindow();
  virtual ~MainWindow();
protected:
  Gtk::VBox vbox;
  Gtk::MenuBar menubar;
  /* IMPORTANT If more menus are added see whether they should be made in(sensitive) depending on whether a project has been opened. */
  Gtk::Menu menufile;
  Gtk::Menu menuedit;
  Gtk::Menu_Helpers::MenuElem menu_edit_undo;
  Gtk::Menu_Helpers::MenuElem menu_edit_redo;
  Gtk::Menu menuview;
  Gtk::CheckMenuItem menuview_backups;
  Gtk::Menu menugoto;
  Gtk::Menu menutools;
  Gtk::Menu menupreferences;
  Gtk::Menu menudiagnostics;
  Gtk::Menu menuhelp;
  Gtk::HBox hbox;
  Gtk::Button backbutton;
  Gtk::Button forwardbutton;
  Gtk::Combo combo_book;
  Gtk::Combo combo_chapter;
  Gtk::Combo combo_verse;
  Gtk::VPaned vpane;
  Gtk::HPaned hpane;
  Gtk::ScrolledWindow scrolledwindow_text;
  Gtk::TextView textview;
  Glib::RefPtr<Gtk::TextTag> fonttag;
  Gtk::ScrolledWindow scrolledwindow_notes;
  Gtk::TextView textview2;
  Gtk::ScrolledWindow scrolledwindow_list;
  Gtk::TreeView treeview;
  class ModelColumns : public Gtk::TreeModel::ColumnRecord {
  public:
    ModelColumns()
    { add(col_name); }
    Gtk::TreeModelColumn<string> col_name;
  };
  ModelColumns columns;
  Glib::RefPtr<Gtk::ListStore> ref_tree_model;

  /* Initialization */
  virtual void initialize_project();
  virtual void enable_or_disable_menus(bool enable);

  /* Menu callbacks */
  virtual void open();
  virtual void close();
  virtual void convert2web();
  virtual void showgeneralscript();
  virtual void showsynchronizationscript();
  virtual void viewlog();
  virtual void showhelp();
  virtual void showabout();
  virtual void synchronization();
  virtual void synchronize_now();
  virtual void synchronize_interactive();
  virtual void menu_undo();
  virtual void menu_redo();
  virtual void menu_edit();
  virtual void menu_find();
  virtual void companion_settings();
  virtual void editor_settings();

  /* Navigation  */
  virtual void on_combo_book_changed();
  virtual void on_combo_chapter_changed();
  virtual void on_combo_verse_changed();
  virtual void goto_next_verse();
  virtual void goto_previous_verse();
  virtual void goto_next_chapter();
  virtual void goto_previous_chapter();
  virtual void goto_next_book();
  virtual void goto_previous_book();
  virtual void go_to_reference(const string& book, const string& chapter, const string& verse, bool jump);
  vector<string> requested_book;
  vector<string> requested_chapter;
  vector<string> requested_verse;
  vector<bool> requested_jump;
  string goto_book, goto_chapter, goto_verse;
  string current_book, current_chapter, current_verse;
  virtual bool on_navigation_timeout();
  bool driving_book, driving_chapter, driving_verse;
  virtual void on_cursor_moved(Gtk::MovementStep step, int count, bool extended_selection);
  bool on_cursor_moved_initialized;
  virtual void on_grab_focus();
  string cursor_get_verse_number();
  unsigned int on_grab_focus_timer;
  string goto_cursor_position;
  unsigned int keypress_timer;

  
  /* Tracker */
  virtual void goto_next_reference();
  virtual void goto_previous_reference();
  virtual bool set_next_or_previous_item(Gtk::Combo& combo, const string& current_text, bool next);
  virtual void set_sensitivity_of_navigator_buttons();
  virtual bool on_tracker_timeout();
  virtual void set_next_or_previous_reference(bool next);
  Track track;

  /* Variables for general use */
  Configuration configuration;
  Stylesheet stylesheet;
  Scripture scripture;

  /* Loading, editing, saving */
  Book book;
  virtual void editor_load_chapter();
  virtual void editor_position_cursor_at_verse(const string& verse);
  virtual void editor_save();
  virtual bool on_save_timeout();

  /* Clipboard */
  virtual void on_cut();
  virtual void on_copy();
  virtual void on_paste();

  /* Backups */
  virtual void on_view_backups();
  Backup backup;
  virtual void view_backups_if_applicable();

  /* List store and reference handling */
  virtual void fill_list_store(vector<string>& lines);
  virtual void on_list_selection_changed();
  virtual void on_open_references();
  virtual void on_save_references();

  /* Synchronization */
  virtual void synchronize_execute();
  virtual bool on_synchronize_timeout();

  /* Undo and Redo */
  Undo undo;
  virtual void on_insert(const Gtk::TextBuffer::iterator& pos, const Glib::ustring& text, int bytes);
  virtual void on_erase(const Gtk::TextBuffer::iterator& range_begin, const Gtk::TextBuffer::iterator& range_end);

  /* Undo and Redo */
  virtual bool on_windows_companion_timeout();
  WindowsCompanion windowscompanionlocal;
  WindowsCompanion windowscompanionremote;
};


#endif


