/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "ssf.h"
#include "scripture.h"
#include "bible.h"



Scripture::Scripture(int dummy) {
}


Scripture::~Scripture() {
}


void Scripture::load(const string& MyParatextProjects, unsigned int index) {
  /*
    This procedures takes the Scripture at "index" of all the available Scriptures.
    It looks up all files, together with the complete path, which are in that Scripture.
    It looks up all English names of the books that are in that Scripture.
    In Paratext 6 language a "Scripture" is a "Paratext Project".
  */

  // Clear the variables that may have been previously used.
  clear();
  // Read the directory with all the projects and select our project.
  ScriptureSettingsFile ssf(MyParatextProjects);
  string shortcut = ssf.shortcuts[index];
  // Search for all books in the project directory.
  string PrePart;
  string PostPart;
  string BookNameForm;
  ssf.Naming (index, PrePart, PostPart, BookNameForm);
  string path = ssf.Directory (index);
  ReadDirectory rd(path, PrePart, PostPart);
  vector<string> v1;
  for (unsigned int i = 0; i < rd.files.size(); i++) {
    v1.push_back(rd.files[i]);
  }
  vector<string> v2;
  for (unsigned int i = 0; i < v1.size(); i++) {
    string s = v1[i];
    s = s.substr(PrePart.length(), 1000);
    s = s.substr(0, s.length() - PostPart.length());
    while (s.length() > 3) s.erase(0, 1);
    v2.push_back(s);
  }
  // Store all the data about the books for later use.
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
    string s = index_to_id (i);
    for (unsigned int i2 = 0; i2 < v2.size(); i2++) {
      if (s == v2[i2]) {
	FilePathAndNames.push_back(MyParatextProjects + "/" + shortcut + "/" + v1[i2]);
	IDs.push_back(v2[i2]);
	EnglishBookNames.push_back(id_to_biblebook_english(v2[i2]));
      }
    }
  }
  // Store the data directory
  bibledit_data_directory = Glib::build_filename (path, "bibledit");
}


void Scripture::clear() {
  EnglishBookNames.clear();
  FilePathAndNames.clear();
  IDs.clear();
  bibledit_data_directory.clear();
}


int Scripture::get_index_of_id (const string& id) {
  // Takes book id, and returns its index in all the books.
  int returnvalue = -1;
  string s = id;
  if (s.length() >= 3) {
    s = s.substr(0, 3);
    for (unsigned int i = 0; i < IDs.size(); i++) {
      if (s == IDs[i]) {
	returnvalue = i;
	break;
      }
    }
  } 
  return returnvalue;
}


string Scripture::get_bibledit_data_directory() {
  // Ensure the directory exists.
  create_directory(bibledit_data_directory);
  // Return the value.
  return bibledit_data_directory;
}


string Scripture::get_backup_directory(int bookindex) {
  // Build the directory.
  string directory = Glib::build_filename (bibledit_data_directory, EnglishBookNames[bookindex] + "/backups");
  // Ensure it exists.
  create_directory(directory);
  // Return the value.
  return directory;
}

