/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include <libgen.h>
#include <gtkmm.h>
#include "synchronize.h"
#include "configuration.h"
#include "log.h"


#define BIBLEDIT "bibledit"


string unison_config_file_name() {
  string s = BIBLEDIT;
  s.append(".prf");
  s = Glib::build_filename(".unison", s);
  return Glib::build_filename(Glib::get_home_dir(), s);
}


void synchronize(bool graphic) {
  if (!graphic) {
    log_message ("Synchronization started");
  }
  string command;
  command.append("unison ");
  command.append(BIBLEDIT);
  command.append(" -ui ");
  if (graphic) command.append("graphic");
  else command.append("text > " + synchronize_temporary_filename() + " 2>&1");
  command.append (" &");
  system(command.c_str());
}


void synchronize_handle_config_file(bool produce, const string& root1, const string& root2) {
  if (produce) {
    WriteText wt(unison_config_file_name());
    wt.out << "# Unison preferences file" << endl;
    wt.out << "root = " << root1 << endl;
    wt.out << "root = " << root2 << endl;
    wt.out << "batch = true" << endl;
    wt.out << "path = none" << endl;
    ReadText rt(synchronization_paths_filename(), true);
    for (unsigned int i = 0; i < rt.lines.size(); i++) {
      string s = trim(rt.lines[i]);
      if (!s.empty()) {
	wt.out << "path = " << rt.lines[i] << endl;
      }
    }
  } else {
    unlink (unison_config_file_name().c_str());
  }
}


string synchronize_temporary_filename() {
  return temporary_file("bibledit.unison");
}
